# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4

from .wiggle import Entity
from fifo.helper import *

network_fmt = {
    'uuid':
    {'title': 'UUID', 'len': 36, 'fmt': '%36s', 'get': lambda e: d(e, ['uuid'])},
    'name':
    {'title': 'Name', 'len': 10, 'fmt': '%-10s', 'get': lambda e: d(e, ['name'])},
    'ipranges':
    {'title': 'ipranges', 'len': 36, 'fmt': '%36s', 'get': lambda e: ','.join(d(e, ['ipranges']))},
 }

def create(args):
    res = args.endpoint.create(args.name)
    if res:
        print 'Network successfully created: %s' % res['uuid']
    else:
        print 'Network creation failed: %r' % res
        exit(1)

def add_range(args):
    res = args.endpoint.add_range(args.uuid, args.iprange_uuid)
    if res:
        print 'Successfully added IP range %s to %s' % (args.iprange_uuid, args.uuid)
    else:
        print 'Adding IP range failed: %r' % res
        exit(1)

class Network(Entity):
    def __init__(self, wiggle):
        self._wiggle = wiggle
        self._resource = 'networks'

    def create(self, name):
        specs = {
            'name': name
        }
        return self._post(specs)

    def add_range(self, network, iprange):
        return self._put_attr(network, 'ipranges/' + iprange, {})

    def make_parser(self, subparsers):
        parser_networks = subparsers.add_parser('networks', help='network related commands')
        parser_networks.set_defaults(endpoint=self)
        subparsers_networks = parser_networks.add_subparsers(help='network commands')
        self.add_metadata_parser(subparsers_networks)
        parser_networks_list = subparsers_networks.add_parser('list', help='lists networks')
        parser_networks_list.add_argument('--fmt', action=ListAction,
                                          default=['uuid', 'name','ipranges'])
        parser_networks_list.add_argument('-H', action='store_false')
        parser_networks_list.add_argument('-p', action='store_true')
        parser_networks_list.set_defaults(func=show_list,
                                          fmt_def=network_fmt)
        parser_networks_get = subparsers_networks.add_parser('get', help='gets a network')
        parser_networks_get.add_argument('uuid')
        parser_networks_get.set_defaults(func=show_get)
        parser_networks_delete = subparsers_networks.add_parser('delete', help='gets a network')
        parser_networks_delete.add_argument('uuid')
        parser_networks_delete.set_defaults(func=show_delete)
        parser_networks_create = subparsers_networks.add_parser('create', help='adds a network')
        parser_networks_create.add_argument('name')
        parser_networks_create.set_defaults(func=create)
        parser_networks_addrange = subparsers_networks.add_parser('add-range', 
                                         help='add an iprange to a network')
        parser_networks_addrange.add_argument('uuid')
        parser_networks_addrange.add_argument('iprange_uuid')
        parser_networks_addrange.set_defaults(func=add_range)



# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4

from .wiggle import Entity
from fifo.helper import *

role_fmt = {
    'uuid':
    {'title': 'UUID', 'len': 36, 'fmt': '%36s', 'get': lambda e: d(e, ['uuid'])},
    'name':
    {'title': 'Name', 'len': 10, 'fmt': '%-10s', 'get': lambda e: d(e, ['name'])},
}

def create(args):
    res = args.endpoint.create(args.name)
    if args.p:
        if res:
            print res['uuid']
        else:
            exit(1)
    else:
        if res:
            print 'Role successfully created: %s' % res['uuid']
        else:
            print 'Role creation failed: %r' % res
            exit(1)

def grant(args):
    res = args.endpoint.grant(args.uuid, args.permission)
    if args.p:
        if res:
            exit(0)
        else:
            exit(1)
    else:
        if res:
            print 'Granted %r to %s' % (args.permission, res)
        else:
            print 'Grant failed: %r' % res
            exit(1)

class Role(Entity):
    def __init__(self, wiggle):
        self._wiggle = wiggle
        self._resource = 'roles'

    def create(self, name):
        return self._post({'name': name})

    def grant(self, role, permission):
        return self._put_attr(role, ['permissions'] + permission, {})

    def make_parser(self, subparsers):
        parser_roles = subparsers.add_parser('roles', help='role related commands')
        parser_roles.set_defaults(endpoint=self)
        subparsers_roles = parser_roles.add_subparsers(help='role commands')
        self.add_metadata_parser(subparsers_roles)
        parser_roles_list = subparsers_roles.add_parser('list', help='lists roles')
        parser_roles_list.add_argument('--fmt', action=ListAction,
                                          default=['uuid', 'name'])
        parser_roles_list.add_argument('-H', action='store_false')
        parser_roles_list.add_argument('-p', action='store_true')
        parser_roles_list.set_defaults(func=show_list,
                                          fmt_def=role_fmt)
        parser_roles_get = subparsers_roles.add_parser('get', help='gets a role')
        parser_roles_get.add_argument('uuid')
        parser_roles_get.set_defaults(func=show_get)
        parser_roles_delete = subparsers_roles.add_parser('delete', help='gets a role')
        parser_roles_delete.add_argument('uuid')
        parser_roles_delete.set_defaults(func=show_delete)
        parser_roles_create = subparsers_roles.add_parser('create', help='creates a role')
        parser_roles_create.add_argument('-p', action='store_true')
        parser_roles_create.add_argument('name')
        parser_roles_create.set_defaults(func=create)
        parser_roles_grant = subparsers_roles.add_parser('grant', help='grants a permission to a role')
        parser_roles_grant.add_argument('-p', action='store_true')
        parser_roles_grant.add_argument('uuid')
        parser_roles_grant.add_argument('permission', nargs='*')
        parser_roles_grant.set_defaults(func=grant)



# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4

from __future__ import print_function
import argparse
import json
import re
from pprint import pprint
import sys

# Verbose print support
# http://stackoverflow.com/questions/5980042/how-to-implement-the-verbose-or-v-option-into-a-script
# below is the 2.x version, may be change in for 3.x version python
_vprint = lambda *a, **k: None

def vprint(*args):
    _vprint("> ", end="", file=sys.stderr)
    _vprint(*args, file=sys.stderr)

def init_vprint(verbose):
    global _vprint
    _vprint = print if verbose else lambda *a, **k: None

# curl print support
_curlprintSwitch = False

def curlprint(host, method, path, headers, data=None, upload=None, file=None, fileMode="ascii"):
    if _curlprintSwitch:
        print("> [curl] ", end="", file=sys.stderr)
        headersp = ' '.join(map(lambda x: '-H "' + x +':' + headers[x] + '"', headers))
        datap = ''
        if data:
            datap = '-d ' + json.dumps(data)
        filep = ''
        if file:
            filep = '--data-' + fileMode + ' @' + file
        uploadp = ''
        if upload:
            uploadp = '-T ' + upload
        print("curl -X {method} http://{host}{path} {headers_params} {data_params} {upload_params} {file_params}".format(
            method=method,
            host=host,
            path=path,
            headers_params=headersp,
            data_params=datap,
            upload_params=uploadp,
            file_params=filep
        ), file=sys.stderr)

def init_curlprint(curl):
    global _curlprintSwitch
    _curlprintSwitch = curl

# We need to add a own action for lists as arguments
class ListAction(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        setattr(namespace, self.dest, values.split(','))

def is_uuid(str):
    regexp = '^[0-9a-fA-F]{8}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{12}$'
    if re.match(regexp, str):
        return True
    else:
        return False

# Gets a value from a nested hash map or returns a given default if the value
# is not present
def d(o, p, deflt="-"):
    if p == []:
        return o
    else:
        k = p[0]
        if k in o:
            return d(o[k], p[1:], deflt)
        else:
            return deflt

# Helper function to generate a formatstring out of the format definition and the selected fields
def mk_fmt_str(args):
    s = ""
    for k in args.fmt:
        s = s + args.fmt_def[k]['fmt'] + " "
    return s

# Helper function to generate the format values for one of the lines.
def mk_fmt_line(args, e):
    r = []
    for k in args.fmt:
        r.append(args.fmt_def[k]['get'](e))
    return r

# Prints the header for a list opperation based on the selected format
def header(args):
    fmt = mk_fmt_str(args)
    hfmt = fmt.replace('d', 's')
    r = []
    for k in args.fmt:
        r.append(args.fmt_def[k]['title'])
    if args.p:
        print(":".join(r))
    else:
        print(hfmt % tuple(r))
    r = []
    if not args.p:
        for k in args.fmt:
            r.append("-" * args.fmt_def[k]['len'])
        print(hfmt % tuple(r))

# Shows the data when list was selected.
def show_list(args):
    l = args.endpoint.list()
    if not l and l != []:
        print("error!")
        exit(1)
    if args.H:
        header(args)
    fmt = mk_fmt_str(args)
    for e in l:
        if not e:
            print("error!")
            exit(1)
        l = mk_fmt_line(args, e)
        if args.p:
            print(":".join(l))
        else:
            print(fmt%tuple(l))

# Shows the data when get was selected, outputs it in JSON
def show_get(args):
    e = args.endpoint.get(args.uuid)
    if not e:
        print("error!")
        exit(1)
    if 'map_fn' in args:
        e = args.map_fn(e)
    print(json.dumps(e, sort_keys=True, indent=2, separators=(',', ': ')))


# Shows the data when get was selected, outputs it in JSON
def show_delete(args):
    if not args.endpoint.delete(args.uuid):
        print("Failed to delete " + args.uuid)
        exit(1)
    print(args.uuid + " deleted successful.")




# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4

from .wiggle import Entity
from fifo.helper import *

hypervisor_fmt = {
    'uuid':
    {'title': 'UUID', 'len': 36, 'fmt': '%36s', 'get': lambda e: d(e, ['uuid'])},
    'name':
    {'title': 'Name', 'len': 30, 'fmt': '%30s', 'get': lambda e: d(e, ['alias'])},
    'free':
    {'title': 'Free RAM', 'len': 10, 'fmt': '%10s',
     'get': lambda e: str(d(e, ['resources', 'free-memory'])) + 'MB'},
    'used':
    {'title': 'Used RAM', 'len': 10, 'fmt': '%10s',
     'get': lambda e: str(d(e, ['resources', 'provisioned-memory'])) + 'MB'},
    'reserved':
    {'title': 'Reserved RAM', 'len': 10, 'fmt': '%10s',
     'get': lambda e: str(d(e, ['resources', 'reserved-memory'])) + 'MB'},
    'total':
    {'title': 'Total RAM', 'len': 10, 'fmt': '%10s',
     'get': lambda e: str(d(e, ['resources', 'total-memory'])) + 'MB'},
    'smartos':
    {'title': 'SmartOS Version', 'len': 20, 'fmt': '%-20s',
     'get': lambda e: d(e, ['sysinfo', 'Live Image'])},
    'ip':
    {'title': 'IP', 'len': 20, 'fmt': '%20s',
     'get': lambda e: d(e, ['host'])},
}

def map_path(path):
    p =  path.split(":")
    return {"name": p[0], "cost": p[1]}

def set_path(args):
    path = map(map_path, args.path)
    res = args.endpoint.set_path(args.uuid, path)
    if res:
        print "Path set."
    else:
        print "Failed to set path:", res

class Hypervisor(Entity):
    def __init__(self, wiggle):
        self._wiggle = wiggle
        self._resource = 'hypervisors'
    def set_path(self, uuid, path):
        Arg = {"path": path}
        return self._put_attr(uuid, 'config', Arg)

    def make_parser(self, subparsers):
        parser_hypervisors = subparsers.add_parser('hypervisors', help='hypervisor related commands')
        parser_hypervisors.set_defaults(endpoint=self)
        subparsers_hypervisors = parser_hypervisors.add_subparsers(help='hypervisor commands')
        self.add_metadata_parser(subparsers_hypervisors)
        parser_hypervisors_list = subparsers_hypervisors.add_parser('list', help='lists hypervisors')
        parser_hypervisors_list.add_argument('--fmt', action=ListAction,
                                             default=['uuid', 'name', 'free', 'used', 'reserved'])
        parser_hypervisors_list.add_argument('-H', action='store_false')
        parser_hypervisors_list.add_argument('-p', action='store_true')
        parser_hypervisors_list.set_defaults(func=show_list,
                                          fmt_def=hypervisor_fmt)
        parser_hypervisors_get = subparsers_hypervisors.add_parser('get', help='gets a hypervisor')
        parser_hypervisors_get.add_argument('uuid')
        parser_hypervisors_get.set_defaults(func=show_get)
        parser_hypervisors_delete = subparsers_hypervisors.add_parser('delete', help='deletes a hypervisor')
        parser_hypervisors_delete.add_argument('uuid')
        parser_hypervisors_delete.set_defaults(func=show_delete)

        parser_hypervisors_path = subparsers_hypervisors.add_parser('path', help='sets a hypervisor')
        parser_hypervisors_path.add_argument('uuid')
        parser_hypervisors_path.add_argument('path', nargs='*')
        parser_hypervisors_path.set_defaults(func=set_path)



# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4

from .wiggle import Entity
from fifo.helper import *


user_fmt = {
    'uuid':
    {'title': 'UUID', 'len': 36, 'fmt': '%36s', 'get': lambda e: d(e, ['uuid'])},
    'name':
    {'title': 'Name', 'len': 10, 'fmt': '%-10s', 'get': lambda e: d(e, ['name'])},
    'roles':
    {'title': 'Roles', 'len': 10, 'fmt': '%-10s', 'get': lambda e: str(d(e, ['roles']))},
    'orgs':
    {'title': 'Orgs', 'len': 10, 'fmt': '%-10s', 'get': lambda e: str(d(e, ['orgs']))},
}



def user_create(args):
    if not args.password:
        print 'You have to specify a passwrd'
        exit(1)
    wiggle = args.endpoint._wiggle
    reply = args.endpoint.create(args.name, args.password)
    if reply:
        if args.organization:
            args.endpoint.join_org(reply['uuid'], args.organization)
        if args.p:
            if reply:
                print reply['uuid']
            else:
                exit(1)
        else:
            if reply:
                print 'User successfully created: %s' % reply['uuid']
            else:
                print 'User creation failed: %r' % reply
                exit(1)
    else:
        print 'Faied to create VM.'

def grant(args):
    res = args.endpoint.grant(args.uuid, args.permission)
    if args.p:
        if res:
            exit(0)
        else:
            exit(1)
    else:
        if res:
            print 'Granted %r to %s' % (args.permission, res)
        else:
            print 'Grant failed: %r' % res
            exit(1)

def user_delete(args):
    args.endpoint.delete(args.endpoint.uuid_by_name(args.uuid))

def sign_csr(args):
    if args.csr:
        f = open(args.csr, 'r')
        csr = f.read()
        f.close()
    else:
        csr = sys.stdin.read()
    uuid = args.endpoint.uuid_by_name(args.uuid)
    comment = args.comment
    #TODO: make scope configurable?
    scope = ["*"]
    r = args.endpoint.sign(uuid, comment, scope, csr)
    if r:
        print r["cert"]
        exit(0)
    else:
        print r
        exit(1)

class User(Entity):
    def __init__(self, wiggle):
        self._wiggle = wiggle
        self._resource = 'users'

    def create(self, name, password):
        return self._post({'user': name,
                           'password': password})

    def grant(self, user, permission):
        return self._put_attr(user, ['permissions'] + permission, {})

    def sign(self, uuid, comment, scope, csr):
        payload = {
            "scope": ["*"],
            "comment": comment,
            "csr": csr
        }
        return self._post_attr(uuid, 'tokens', payload)

    def join_org(self, uuid, org):
        return self._put_attr(uuid, 'orgs/' + org, {})

    def make_parser(self, subparsers):
        parser_users = subparsers.add_parser('users', help='user related commands')
        parser_users.set_defaults(endpoint=self)
        subparsers_users = parser_users.add_subparsers(help='user commands')
        self.add_metadata_parser(subparsers_users)
        parser_users_list = subparsers_users.add_parser('list', help='lists users')
        parser_users_list.add_argument('--fmt', action=ListAction,
                                       default=['uuid', 'name'])
        parser_users_list.add_argument('-H', action='store_false')
        parser_users_list.add_argument('-p', action='store_true')
        parser_users_list.set_defaults(func=show_list,
                                       fmt_def=user_fmt)
        parser_users_get = subparsers_users.add_parser('get', help='gets a user')
        parser_users_get.add_argument('uuid')
        parser_users_get.set_defaults(func=show_get)
        parser_users_sign = subparsers_users.add_parser('sign', help='sign a CSR and links the certificate.')
        parser_users_sign.add_argument('uuid', help="UUID of the user")
        parser_users_sign.add_argument('--csr', help="CSR file from openssl req command")
        parser_users_sign.add_argument('--comment', help="comment", default="SSL Cert")
        parser_users_sign.set_defaults(func=sign_csr)

        parser_users_delete = subparsers_users.add_parser('delete', help='gets a user')
        parser_users_delete.add_argument('uuid')
        parser_users_delete.set_defaults(func=show_delete)
        # fifo users create bill -p pass -g role -o org
        parser_users_create = subparsers_users.add_parser('create', help='creates a new user')
        parser_users_create.add_argument('name',
                                         help='Name of the user')
        parser_users_create.add_argument('-p', action='store_true')
        parser_users_create.add_argument('--password', '-P',
                                         help='Password of the user.')
        parser_users_create.add_argument('--role', '-g',
                                         help='Role of the user.')
        parser_users_create.add_argument('--organization', '-o',
                                         help='Organization of the user.')
        parser_users_create.set_defaults(func=user_create)
        parser_users_delete = subparsers_users.add_parser('delete', help='deletes a user')
        parser_users_delete.add_argument('uuid',
                                         help='uuid of VM to show')
        parser_users_delete.set_defaults(func=user_delete)
        parser_users_grant = subparsers_users.add_parser('grant', help='grants a permission to a user')
        parser_users_grant.add_argument('-p', action='store_true')
        parser_users_grant.add_argument('uuid')
        parser_users_grant.add_argument('permission', nargs='*')
        parser_users_grant.set_defaults(func=grant)



# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4

from .wiggle import Entity
from fifo.helper import *


dtrace_fmt = {
    'uuid':
    {'title': 'UUID', 'len': 36, 'fmt': '%36s', 'get': lambda e: d(e, ['uuid'])},
    'name':
    {'title': 'Name', 'len': 40, 'fmt': '%-40s', 'get': lambda e: d(e, ['name'])},
}


class Dtrace(Entity):
    def __init__(self, wiggle):
        self._wiggle = wiggle
        self._resource = 'dtrace'

    def make_parser(self, subparsers):
        parser_dtraces = subparsers.add_parser('dtrace', help='dtrace related commands')
        parser_dtraces.set_defaults(endpoint=self)
        subparsers_dtraces = parser_dtraces.add_subparsers(help='dtrace commands')
        self.add_metadata_parser(subparsers_dtraces)
        parser_dtraces_list = subparsers_dtraces.add_parser('list', help='lists dtraces')
        parser_dtraces_list.add_argument('--fmt', action=ListAction,
                                          default=['uuid', 'name'])
        parser_dtraces_list.add_argument('-H', action='store_false')
        parser_dtraces_list.add_argument('-p', action='store_true')
        parser_dtraces_list.set_defaults(func=show_list,
                                          fmt_def=dtrace_fmt)
        parser_dtraces_get = subparsers_dtraces.add_parser('get', help='gets a dtrace')
        parser_dtraces_get.add_argument('uuid')
        parser_dtraces_get.set_defaults(func=show_get)
        parser_dtraces_delete = subparsers_dtraces.add_parser('delete', help='gets a dtrace')
        parser_dtraces_delete.add_argument('uuid')
        parser_dtraces_delete.set_defaults(func=show_delete)



# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4

from .wiggle import Entity
from fifo.helper import *

iprange_fmt = {
    'uuid':
    {'title': 'UUID', 'len': 36, 'fmt': '%36s', 'get': lambda e: d(e, ['uuid'])},
    'name':
    {'title': 'Name', 'len': 20, 'fmt': '%-20s', 'get': lambda e: d(e, ['name'])},
    'tag':
    {'title': 'Tag', 'len': 10, 'fmt': '%-10s', 'get': lambda e: d(e, ['tag'])},
    'iprange':
    {'title': 'Iprange', 'len': 15, 'fmt': '%15s', 'get': lambda e: d(e, ['iprange'])},
    'first':
    {'title': 'First', 'len': 15, 'fmt': '%15s', 'get': lambda e: d(e, ['first'])},
    'next':
    {'title': 'Next', 'len': 15, 'fmt': '%15s', 'get': lambda e: d(e, ['current'])},
    'last':
    {'title': 'Last', 'len': 15, 'fmt': '%15s', 'get': lambda e: d(e, ['last'])},
    'returned':
    {'title': 'Returned', 'len': 15, 'fmt': '%15s', 'get': lambda e: len(d(e, ['free'], []))},
    'gateway':
    {'title': 'Gateway', 'len': 15, 'fmt': '%15s', 'get': lambda e: d(e, ['gateway'])},
    'netmask':
    {'title': 'Netmask', 'len': 15, 'fmt': '%15s', 'get': lambda e: d(e, ['netmask'])},
    'vlan':
    {'title': 'vlan', 'len': 5, 'fmt': '%5s', 'get': lambda e: d(e, ['vlan'])},
}

def create(args):

    if args.vlan:
        res = args.endpoint.create(args.name, args.network, args.netmask, args.gateway,
                                args.first, args.last, args.tag, int(args.vlan))
    else:
        res = args.endpoint.create(args.name, args.network, args.netmask, args.gateway, 
                                args.first, args.last, args.tag)
    if res:
        print 'IP range successfully created: %s' % res['uuid']
    else:
        print 'IP range creation failed: %r' % res
        exit(1)

class Iprange(Entity):
    def __init__(self, wiggle):
        self._wiggle = wiggle
        self._resource = 'ipranges'

    def create(self, name, network, netmask, gateway, first, last, nic_tag, vlan=None):
        specs = {
            'name': name,
            'network': network,
            'netmask': netmask,
            'gateway': gateway,
            'first': first,
            'last': last,
            'tag': nic_tag,
        }
        if vlan > 0:
            specs['vlan'] = vlan

        return self._post(specs)

    def make_parser(self, subparsers):
        parser_ipranges = subparsers.add_parser('ipranges', help='iprange related commands')
        parser_ipranges.set_defaults(endpoint=self)
        subparsers_ipranges = parser_ipranges.add_subparsers(help='iprange commands')
        self.add_metadata_parser(subparsers_ipranges)
        parser_ipranges_list = subparsers_ipranges.add_parser('list', help='lists ipranges')
        parser_ipranges_list.add_argument('--fmt', action=ListAction,
                                          default=['uuid', 'name', 'tag', 'first', 'last', 'vlan'])
        parser_ipranges_list.add_argument('-H', action='store_false')
        parser_ipranges_list.add_argument('-p', action='store_true')
        parser_ipranges_list.set_defaults(func=show_list,
                                          fmt_def=iprange_fmt)
        parser_ipranges_get = subparsers_ipranges.add_parser('get', help='gets an iprange')
        parser_ipranges_get.add_argument('uuid')
        parser_ipranges_get.set_defaults(func=show_get)
        parser_ipranges_delete = subparsers_ipranges.add_parser('delete', help='deletes an iprange')
        parser_ipranges_delete.add_argument('uuid')
        parser_ipranges_delete.set_defaults(func=show_delete)
        parser_ipranges_create = subparsers_ipranges.add_parser('create', help='creates an ip range')
        parser_ipranges_create.add_argument('name')
        parser_ipranges_create.add_argument('--network', '-n', required=True, help='Network for the iprange.')
        parser_ipranges_create.add_argument('--netmask', '-m', required=True, help='Netmask for the iprange.')
        parser_ipranges_create.add_argument('--gateway', '-g', required=True, help='Gateway for the iprange.')
        parser_ipranges_create.add_argument('--first', '-f', required=True, help='First assignable IP address for the iprange.')
        parser_ipranges_create.add_argument('--last', '-l', required=True, help='Last assignable IP address for the iprange.')
        parser_ipranges_create.add_argument('--tag', '-t', required=True,
                                help='NIC Tag to use for the iprange. This must exist on at least one hypervisor')
        parser_ipranges_create.add_argument('--vlan', '-v', type=int, help='Vlan for the iprange.')
        parser_ipranges_create.set_defaults(func=create)



# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4

from .wiggle import Entity
from .package import Package
from .dataset import Dataset

from fifo.helper import *
from fifo.api.package import *
from fifo.api.dataset import *

from datetime import datetime
import sys
import json

def show_services(args):
    e = args.endpoint.get(args.uuid)
    if not e or not e.has_key('services'):
        exit(1)
    services = e['services']
    if args.j:
        print(json.dumps(services, sort_keys=True, indent=2, separators=(',', ': ')))
    else:
        if args.H:
            print('%-20s %s\n' % ('State', 'Service')),
        for service in services.keys():
            state = services[service]
            if args.a or state != 'disabled':
                if args.p:
                    print('%s\t%s' % (state, service))
                else:
                    print('%-20s %s' % (state, service))

def svcadm_action(args):
    if args.endpoint.service_action(args.uuid, args.action, args.service):
        print('Command issued to %s service %s on %s.' % (args.action, args.service, args.uuid))
    else:
        print('Failed to %s service %s on %s.' % (args.action, args.service, args.uuid))
        exit(1)


def vm_action(args):
    if args.action == 'start':
        args.endpoint.start(args.uuid)

    elif args.action == 'stop':
        if args.f:
            args.endpoint.force_stop(args.uuid)
        else:
            args.endpoint.stop(args.uuid)
    elif args.action == 'reboot':
        if args.f:
            args.endpoint.force_reboot(args.uuid)
        else:
            args.endpoint.reboot(args.uuid)

def snapshot_create(args):
    res = args.endpoint.make_snapshot(args.vmuuid, args.comment)
    if res:
        print 'Snapshot successfully created!'
    else:
        print 'Snapshot creation failed!'

def backup_create(args):
    res = args.endpoint.make_backup(args.vmuuid, args.comment, args.d, args.parent)
    if res:
        print 'Snapshot successfully created!'
    else:
        print 'Snapshot creation failed!'

# Helper functions to format the different getters for VM's
def vm_map_fn(vm):
    return(vm['config'])

def vm_raw_map_fn(vm):
    return(vm)

def vm_info_map_fn(vm):
    return(vm['info'])

def vm_metadata_map_fn(vm):
    return(vm['metadata'])

# Returns the ip of a vm (first ip in the networks)
def vm_ip(e):
    n = d(e, ['config', 'networks'], [])
    if len(n) > 0:
        return n[0]['ip']
    else:
        return '-'

vm_fmt = {
    'uuid':
    {'title': 'UUID', 'len': 36, 'fmt': '%36s', 'get': lambda e: d(e, ['uuid'])},
    'alias':
    {'title': 'alias', 'len': 10, 'fmt': '%-10s', 'get': lambda e: d(e, ['config', 'alias'])},
    'ip':
    {'title': 'IP', 'len': 15, 'fmt': '%15s', 'get': vm_ip},
    'state':
    {'title': 'state', 'len': 15, 'fmt': '%-15s', 'get': lambda e: d(e, ['state'])},
    'hypervisor':
    {'title': 'hypervisor', 'len': 20, 'fmt': '%-20s', 'get': lambda e: d(e, ['hypervisor'])},
    'package':
    {'title': 'package', 'len': 36, 'fmt': '%36s', 'get': lambda e: d(e, ['package'])},
    'dataset':
    {'title': 'dataset', 'len': 36, 'fmt': '%36s', 'get': lambda e: d(e, ['config', 'dataset'])},
}

snapshot_fmt = {
    'uuid':
    {'title': 'UUID', 'len': 36, 'fmt': '%36s', 'get': lambda e: d(e, ['uuid'])},
    'timestamp':
    {'title': 'Timestamp', 'len': 20, 'fmt': '%-20s',
     'get': lambda e: datetime.fromtimestamp(d(e, ['timestamp'])/1000000).isoformat()},
    'size':
    {'title': 'Size', 'len': 5, 'fmt': '%5d', 'get': lambda e: d(e, ['size'])},
    'comment':
    {'title': 'Comment', 'len': 30, 'fmt': '%-30s', 'get': lambda e: d(e, ['comment'])},
}

backup_fmt = {
    'uuid':
    {'title': 'UUID', 'len': 36, 'fmt': '%36s', 'get': lambda e: d(e, ['uuid'])},
    'parent':
    {'title': 'Parent', 'len': 36, 'fmt': '%36s', 'get': lambda e: d(e, ['parent'])},
    'local':
    {'title': 'Local', 'len': 5, 'fmt': '%5s', 'get': lambda e: 'Yes' if d(e, ['local']) else 'No'},
    'size':
    {'title': 'Size', 'len': 15, 'fmt': '%5d', 'get': lambda e: d(e, ['size'], 0)},
    'state':
    {'title': 'State', 'len': 10, 'fmt': '%10s', 'get': lambda e: d(e, ['state'])},
    'timestamp':
    {'title': 'Timestamp', 'len': 20, 'fmt': '%-20s',
     'get': lambda e: datetime.fromtimestamp(d(e, ['timestamp'])/1000000).isoformat()},
    'comment':
    {'title': 'Comment', 'len': 30, 'fmt': '%-30s', 'get': lambda e: d(e, ['comment'])},
}

def vm_delete(args):
    if args.l:
        args.endpoint.delete_body(args.uuid, {'location':'hypervisor'})
    else:
        args.endpoint.delete(args.uuid)

def vm_create(args):
    if args.file:
        f = open(args.file, 'r')
        config = json.loads(f.read())
        f.close()
    else:
        config = json.loads(sys.stdin.read())

    wiggle = args.endpoint._wiggle

    package = Package(wiggle).uuid_by_name(args.package)
    if not package:
        print 'Could not find package: ' + args.package + '.'
        exit(1)

    dataset = Dataset(wiggle).uuid_by_name(args.dataset)
    if not dataset:
        print 'Could not find dataset: ' + args.dataset + '.'
        exit(1)

    if args.dryrun:
        reply = args.endpoint.dryrun(package, dataset, config)
    else:
        reply = args.endpoint.create(package, dataset, config)

    if reply == True:
        print 'VM dry run succesful'
    elif reply:
        print 'VM ' + reply['uuid'] + ' created successfully.'
    else:
        print 'Failed to create VM.'

# Shows the data when list was selected.
def snapshots_list(args):
    l = args.endpoint.list_snapshots(args.vmuuid)
    if args.H:
        header(args)
    fmt = mk_fmt_str(args)
    for e in l:
        if not e:
            print('error!')
            exit(1)
        l = mk_fmt_line(args, e)
        if args.p:
            print(':'.join(l))
        else:
            print(fmt%tuple(l))

def backups_list(args):
    l = args.endpoint.list_backups(args.vmuuid)
    l = sorted(l, key=lambda e: d(e, ['timestamp'], 0))
    if args.H:
        header(args)
    fmt = mk_fmt_str(args)
    for e in l:
        if not e:
            print('error!')
            exit(1)
        l = mk_fmt_line(args, e)
        if args.p:
            print(':'.join(l))
        else:
            print(fmt%tuple(l))


def snapshot_get(args):
    e = args.endpoint.get_snapshot(args.vmuuid, args.snapuuid)
    if not e:
        print('error!')
        exit(1)
    if 'map_fn' in args:
        e = args.map_fn(e)
    print(json.dumps(e, sort_keys=True, indent=2, separators=(',', ': ')))

def snapshot_delete(args):
    e = args.endpoint.delete_snapshot(args.vmuuid, args.snapuuid)
    if not e:
        print('error!')
        exit(1)
        print 'Snapshot deleted successfully.'

def snapshot_rollback(args):
    e = args.endpoint.rollback_snapshot(args.vmuuid, args.snapuuid)
    if not e:
        print('error!')
        exit(1)
        print 'Snapshot deleted successfully.'

def backup_get(args):
    e = args.endpoint.get_backup(args.vmuuid, args.snapuuid)
    if not e:
        print('error!')
        exit(1)
    if 'map_fn' in args:
        e = args.map_fn(e)
    print(json.dumps(e, sort_keys=True, indent=2, separators=(',', ': ')))

def backup_delete(args):
    e = args.endpoint.delete_backup(args.vmuuid, args.snapuuid, args.l)
    if not e:
        print('error!')
        exit(1)
        print 'Snapshot deleted successfully.'

def backup_restore(args):
    e = args.endpoint.restore_backup(args.vmuuid, args.snapuuid, args.hypervisor)
    if not e:
        print('error!')
        exit(1)
        print 'Snapshot deleted successfully.'

class VM(Entity):
    def __init__(self, wiggle):
        self._wiggle = wiggle
        self._resource = 'vms'
        self._fields = ['uuid', 'config', 'state', 'hypervisor', 'package']

    def name_of(self, obj):
        return d(obj, ['config', 'alias'])

    def create(self, package, dataset, config):
        return self._post({'package': package,
                           'dataset': dataset,
                           'config': config})

    def dryrun(self, package, dataset, config):
        return self._wiggle.put(self._resource,'dry_run',
                                 body={'package': package,
                                       'dataset': dataset,
                                       'config': config})

    def start(self, uuid):
        return self._put_attr(uuid, 'state', {'action': 'start'})

    def stop(self, uuid):
        return self._put_attr(uuid, 'state', {'action': 'stop'})

    def reboot(self, uuid):
        return self._put_attr(uuid, 'state', {'action': 'reboot'})

    def force_stop(self, uuid):
        return self._put_attr(uuid, 'state', {'action': 'stop', 'force': True})

    def force_reboot(self, uuid):
        return self._put_attr(uuid, 'state', {'action': 'reboot', 'force': True})

    def list_snapshots(self, uuid):
        r = self.get(uuid)
        if r:
            if r['snapshots'] != []:
                # This is fucking ugly!
                map(lambda e: e[1].update({'uuid': e[0]}), r['snapshots'].items())
                return r['snapshots'].values()
            else:
                return []
        else:
            return r

    def make_snapshot(self, uuid, comment):
        return self._post_attr(uuid, 'snapshots', {'comment': comment})

    def service_action(self, uuid, action, service):
        return self._put_attr(uuid, 'services', {'action': action, 'service': service})

    def get_snapshot(self, uuid, snapid):
        r = self.get(uuid)
        if r and r != []:
            return r['snapshots'][snapid]
        else:
            return r

    def delete_snapshot(self, uuid, snapid):
        return self._delete_attr(uuid, 'snapshots/' + snapid)

    def rollback_snapshot(self, uuid, snapid):
        return self._put_attr(uuid, 'snapshots/' + snapid, {'action':'rollback'})

    def list_backups(self, uuid):
        r = self.get(uuid)
        if r:
            if r['backups'] != []:
                # This is fucking ugly!
                map(lambda e: e[1].update({'uuid': e[0]}), r['backups'].items())
                return r['backups'].values()
            else:
                return []
        else:
            return r

    def get_backup(self, uuid, snapid):
        r = self.get(uuid)
        if r:
            return r['backups'][snapid]
        else:
            return r

    def delete_backup(self, uuid, snapid, local):
        if local:
            return self._delete_attr_body(uuid, 'backups/' + snapid, {'location':'hypervisor'})
        else:
            return self._delete_attr(uuid, 'backups/' + snapid)

    def restore_backup(self, uuid, snapid, hypervisor):
        if hypervisor:
            Arg = {'action':'rollback', 'hypervisor': hypervisor}
        else:
            Arg = {'action':'rollback'}
        return self._put_attr(uuid, 'backups/' + snapid, Arg)


    def make_backup(self, uuid, comment, delete, parent):
        if parent:
            Arg = {'comment': comment,
                   'delete': delete,
                   'parent': parent}
        else:
            Arg = {'comment': comment,
                   'delete': delete}
        return self._post_attr(uuid, 'backups', Arg)


    def make_parser(self, subparsers):
        parser_vms = subparsers.add_parser('vms', help='vm related commands')
        parser_vms.set_defaults(endpoint=self)
        subparsers_vms = parser_vms.add_subparsers(help='vm commands')
        self.add_metadata_parser(subparsers_vms)
        parser_vms_list = subparsers_vms.add_parser('list', help='lists a vm')
        parser_vms_list.add_argument('--fmt',
                                     action=ListAction, default=['uuid', 'state', 'alias'],
                                     help='Rows to show, valid options are: uuid, alias, ip, state, hypervisor')
        parser_vms_list.add_argument('-H', action='store_false',
                                     help='Supress the header.')
        parser_vms_list.add_argument('-p', action='store_true',
                                     help='show in parsable format, rows sepperated by colon.')
        parser_vms_list.set_defaults(func=show_list,
                                     fmt_def=vm_fmt)
        parser_vms_get = subparsers_vms.add_parser('get', help='gets a VM')
        parser_vms_get.add_argument('--raw', '-r', dest='fmt_def', action='store_const',
                                    const=vm_raw_map_fn, default=vm_map_fn,
                                    help='print the raw result not the config version')
        parser_vms_get.add_argument('uuid',
                                    help='uuid of VM to show')
        parser_vms_get.set_defaults(func=show_get)

        parser_vms_svcs = subparsers_vms.add_parser('svcs', help='shows service states on a VM')
        parser_vms_svcs.add_argument('-H', action='store_false',
                                         help='Supress the header.')
        parser_vms_svcs.add_argument('-p', action='store_true',
                                     help='show in parsable format, rows sepperated by tab.')
        parser_vms_svcs.add_argument('-j', action='store_true',
                                     help='show in json.')
        parser_vms_svcs.add_argument('-a', action='store_true',
                                     help='Show disabled services.')
        parser_vms_svcs.add_argument('uuid',
                                     help='uuid of Services on a VM')

        parser_vms_svcs.set_defaults(func=show_services)

        parser_vms_svcadm = subparsers_vms.add_parser('svcadm', help='service administration tasks')
        parser_vms_svcadm.add_argument('uuid',
                                       help='uuid of Services on a VM')

        subparsers_svcadm = parser_vms_svcadm.add_subparsers(help='svcadm commands')

        parser_svcadm_enable = subparsers_svcadm.add_parser('enable', help='enables a service')
        parser_svcadm_enable.set_defaults(action='enable')
        parser_svcadm_enable.set_defaults(func=svcadm_action)
        parser_svcadm_enable.add_argument('service', help='Service to enable')

        parser_svcadm_disable = subparsers_svcadm.add_parser('disable', help='enables a service')
        parser_svcadm_disable.set_defaults(action='disable')
        parser_svcadm_disable.set_defaults(func=svcadm_action)
        parser_svcadm_disable.add_argument('service', help='Service to disable')

        parser_svcadm_clear = subparsers_svcadm.add_parser('clear', help='enables a service')
        parser_svcadm_clear.set_defaults(action='clear')
        parser_svcadm_clear.set_defaults(func=svcadm_action)
        parser_svcadm_clear.add_argument('service', help='Service to en or clear')

        parser_vms_delete = subparsers_vms.add_parser('delete', help='deletes a VM')
        parser_vms_delete.add_argument('-l', action='store_true', default=False,
                                       help='Delete the vm only on the hypervisor.')
        parser_vms_delete.add_argument('uuid',
                                       help='uuid of VM to show')
        parser_vms_delete.set_defaults(func=vm_delete)

        parser_vms_create = subparsers_vms.add_parser('create', help='creates a new VM')
        parser_vms_create.add_argument('--dryrun', action='store_true', default=False,
                                       help='Does not create the machine but simulates if it can be created.')
        parser_vms_create.add_argument('--package', '-p',
                                       help='UUID of the package to use.')
        parser_vms_create.add_argument('--dataset', '-d',
                                       help='UUID of the dataset to use')
        parser_vms_create.add_argument('--file', '-f',
                                       help='Filename of config.json, if not present will be read from STDIN.')
        parser_vms_create.set_defaults(func=vm_create)

        parser_vms_info = subparsers_vms.add_parser('info', help='gets a vm info')
        parser_vms_info.add_argument('uuid',
                                     help='uuid of VM to show')
        parser_vms_info.set_defaults(func=show_get,
                                     map_fn=vm_info_map_fn)
        parser_vms_start = subparsers_vms.add_parser('start', help='starts a vm')
        parser_vms_start.add_argument('uuid',
                                      help='uuid of VM to start')
        parser_vms_start.set_defaults(func=vm_action,
                                      action='start')
        parser_vms_stop = subparsers_vms.add_parser('stop', help='stops a vm')
        parser_vms_stop.add_argument('uuid',
                                     help='uuid of VM to stop')
        parser_vms_stop.add_argument('-f', action='store_true')
        parser_vms_stop.set_defaults(func=vm_action,
                                     action='stop')
        parser_vms_reboot = subparsers_vms.add_parser('reboot', help='reboot a vm')
        parser_vms_reboot.add_argument('uuid',
                                       help='uuid of VM to reboot')
        parser_vms_reboot.add_argument('-f', action='store_true')
        parser_vms_reboot.set_defaults(func=vm_action,
                                       action='reboot')
        parser_snapshots = subparsers_vms.add_parser('snapshots', help='snapshot related commands')
        parser_snapshots.add_argument('vmuuid',
                                      help='UUID of the VM to work with.')
        subparsers_snapshots = parser_snapshots.add_subparsers(help='snapshot commands')
        parser_snapshots_list = subparsers_snapshots.add_parser('list', help='lists snapshots')
        parser_snapshots_list.add_argument('--fmt', action=ListAction,
                                           default=['uuid', 'timestamp', 'comment'],
                                           help='Fields to show in the list, valid chances are: uuid, timestamp, comment, hypervisor, dataset and package')
        parser_snapshots_list.add_argument('-H', action='store_false',
                                           help='Supress the header.')
        parser_snapshots_list.add_argument('-p', action='store_true',
                                           help='show in parsable format, rows sepperated by colon.')
        parser_snapshots_list.set_defaults(func=snapshots_list,
                                           fmt_def=snapshot_fmt)
        parser_snapshots_get = subparsers_snapshots.add_parser('get', help='gets snapshots')
        parser_snapshots_get.add_argument('snapuuid',
                                          help='UUID if the snapshot')
        parser_snapshots_get.set_defaults(func=snapshot_get)
        parser_snapshots_delete = subparsers_snapshots.add_parser('delete', help='deletes snapshots')
        parser_snapshots_delete.add_argument('snapuuid',
                                             help='UUID if the snapshot')
        parser_snapshots_delete.set_defaults(func=snapshot_delete)
        parser_snapshots_rollback = subparsers_snapshots.add_parser('rollback', help='rolls back a snapshot')
        parser_snapshots_rollback.add_argument('snapuuid',
                                               help='UUID if the snapshot')
        parser_snapshots_rollback.set_defaults(func=snapshot_rollback)
        parser_snapshots_create = subparsers_snapshots.add_parser('create', help='gets snapshots')
        parser_snapshots_create.add_argument('comment',
                                             help='Comment for the snapshot.')
        parser_snapshots_create.set_defaults(func=snapshot_create)
        parser_backups = subparsers_vms.add_parser('backups', help='backup related commands')
        parser_backups.add_argument('vmuuid',
                                    help='UUID of the VM to work with.')
        subparsers_backups = parser_backups.add_subparsers(help='backup commands')
        parser_backups_list = subparsers_backups.add_parser('list', help='lists backups')
        parser_backups_list.add_argument('--fmt', action=ListAction,
                                         default=['uuid', 'local', 'timestamp', 'comment'],
                                         help='Fields to show in the list, valid chances are: uuid, timestamp, comment')
        parser_backups_list.add_argument('-H', action='store_false',
                                         help='Supress the header.')
        parser_backups_list.add_argument('-p', action='store_true',
                                         help='show in parsable format, rows sepperated by colon.')
        parser_backups_list.set_defaults(func=backups_list,
                                         fmt_def=backup_fmt)
        parser_backups_get = subparsers_backups.add_parser('get', help='gets backups')
        parser_backups_get.add_argument('snapuuid',
                                        help='UUID if the backup')
        parser_backups_get.set_defaults(func=backup_get)
        parser_backups_delete = subparsers_backups.add_parser('delete', help='deletes backups')
        parser_backups_delete.add_argument('snapuuid',
                                           help='UUID if the backup')
        parser_backups_delete.add_argument('-l', action='store_true', default=False,
                                           help='Delete only local version of the VM.')
        parser_backups_delete.set_defaults(func=backup_delete)
        parser_backups_restore = subparsers_backups.add_parser('restore', help='rolls back a backup')
        parser_backups_restore.add_argument('--hypervisor', default=False,
                                            help='Restore to a specific hypervisor.')
        parser_backups_restore.add_argument('snapuuid',
                                             help='UUID if the backup')
        parser_backups_restore.set_defaults(func=backup_restore)
        parser_backups_create = subparsers_backups.add_parser('create', help='gets backups')
        parser_backups_create.add_argument('--parent', '-p', default=False,
                                           help='The parent of the backup.')
        parser_backups_create.add_argument('comment',
                                           help='Comment for the backup.')
        parser_backups_create.add_argument('-d', action='store_true', default=False,
                                           help='Delete the backup (or parent) after uploading.')
        parser_backups_create.add_argument('--xml', '-x', action='store_true', default=False,
                                           help='Save the VM\'s .xml for restoring.')

        parser_backups_create.set_defaults(func=backup_create)



# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4

from .wiggle import Entity
from fifo.helper import *

org_fmt = {
    'uuid':
    {'title': 'UUID', 'len': 36, 'fmt': '%36s', 'get': lambda e: d(e, ['uuid'])},
    'name':
    {'title': 'Name', 'len': 10, 'fmt': '%-10s', 'get': lambda e: d(e, ['name'])},
 }

def create(args):
    res = args.endpoint.create(args.name)
    if args.p:
        if res:
            print res['uuid']
        else:
            exit(1)
    else:
        if res:
            print 'Org successfully created: %s' % res['uuid']
        else:
            print 'Org creation failed: %r' % res
            exit(1)

class Org(Entity):
    def __init__(self, wiggle):
        self._wiggle = wiggle
        self._resource = 'orgs'

    def create(self, name):
        return self._post({'name': name})

    def make_parser(self, subparsers):
        parser_orgs = subparsers.add_parser('orgs', help='org related commands')
        parser_orgs.set_defaults(endpoint=self)
        subparsers_orgs = parser_orgs.add_subparsers(help='org commands')
        self.add_metadata_parser(subparsers_orgs)
        parser_orgs_list = subparsers_orgs.add_parser('list', help='lists orgs')
        parser_orgs_list.add_argument('--fmt', action=ListAction,
                                          default=['uuid', 'name'])
        parser_orgs_list.add_argument('-H', action='store_false')
        parser_orgs_list.add_argument('-p', action='store_true')
        parser_orgs_list.set_defaults(func=show_list,
                                          fmt_def=org_fmt)
        parser_orgs_get = subparsers_orgs.add_parser('get', help='gets a org')
        parser_orgs_get.add_argument('uuid')
        parser_orgs_get.set_defaults(func=show_get)
        parser_orgs_delete = subparsers_orgs.add_parser('delete', help='gets a org')
        parser_orgs_delete.add_argument('uuid')
        parser_orgs_delete.set_defaults(func=show_delete)
        parser_orgs_create = subparsers_orgs.add_parser('create', help='creates a Org')
        parser_orgs_create.add_argument('-p', action='store_true')
        parser_orgs_create.add_argument('name')
        parser_orgs_create.set_defaults(func=create)



# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4

import sys
from .wiggle import Entity
from fifo.helper import *
import httplib
import json
import argparse

dataset_fmt = {
    'uuid':
    {'title': 'UUID', 'len': 36, 'fmt': '%36s', 'get': lambda e: d(e, ['uuid'])},
    'name':
    {'title': 'Name', 'len': 10, 'fmt': '%-10s', 'get': lambda e: d(e, ['name'])},
    'version':
    {'title': 'Version', 'len': 7, 'fmt': '%-7s', 'get': lambda e: d(e, ['version'])},
    'type':
    {'title': 'Type', 'len': 5, 'fmt': '%-5s', 'get': lambda e: d(e, ['type'])},
    'description':
    {'title': 'Description', 'len': 10, 'fmt': '%-30s', 'get': lambda e: d(e, ['description'])},
}

def _write_bindata(response, f):
    amt = 4096
    try:
        data = response.read(amt)
    except httplib.IncompleteRead, e:
        data = e.partial
        f.write(data)
        f.flush()
        return

    while data:
        f.write(data)
        f.flush()
        try:
            data = response.read(amt)
        except httplib.IncompleteRead, e:
            vprint('Incomplete read captured')
            data = e.partial

def dataset_get(args):
    response = args.endpoint.get_bindata(args.uuid, '/dataset.gz')
    if response:
        vprint('Headers:', response.getheaders())
        _write_bindata(response, args.out or sys.stdout)

def import_dataset(args):
    j = json.load(args.manifest)
    uuid = j['uuid']
    if not args.dataset_only:
        response = args.endpoint._put_attr(uuid, [], j)
        if response:
            print 'Manifest imported'
        else:
            print 'Manifest upload failed.'
            exit(1)

    print 'Uploading dataset now ...'
    response = args.endpoint._put_file([uuid],  '/dataset.gz', args.dataset)
    if response == 204:
        print 'Import complete!'
    else:
        print 'Dataset upload failed.'
        exit(1)

class Dataset(Entity):
    def __init__(self, wiggle):
        self._wiggle = wiggle
        self._resource = 'datasets'

    def name_of(self, obj):
        return obj['name'] + '-' + obj['version']

    def make_parser(self, subparsers):
        parser_datasets = subparsers.add_parser('datasets', help='dataset related commands')
        parser_datasets.set_defaults(endpoint=self)
        subparsers_datasets = parser_datasets.add_subparsers(help='dataset commands')
        self.add_metadata_parser(subparsers_datasets)
        parser_datasets_list = subparsers_datasets.add_parser('list', help='lists datasets')
        parser_datasets_list.add_argument('--fmt', action=ListAction,
                                          default=['uuid', 'name', 'version', 'type', 'description'])
        parser_datasets_list.add_argument('-H', action='store_false')
        parser_datasets_list.add_argument('-p', action='store_true')
        parser_datasets_list.set_defaults(func=show_list,
                                          fmt_def=dataset_fmt)
        parser_datasets_get = subparsers_datasets.add_parser('get', help='gets a dataset')
        parser_datasets_get.add_argument('uuid')
        parser_datasets_get.set_defaults(func=show_get)
        parser_datasets_delete = subparsers_datasets.add_parser('delete', help='gets a dataset')
        parser_datasets_delete.add_argument('uuid')
        parser_datasets_delete.set_defaults(func=show_delete)
        parser_dataset_import = subparsers_datasets.add_parser('import', help='Imports the binary part of a dataset')
        parser_dataset_import.add_argument('--manifest', '-m', help='manifest to upload',
                                           type=argparse.FileType('r'))
        parser_dataset_import.add_argument('--dataset', '-d', help='dataset to upload',
                                           type=argparse.FileType('r'))
        parser_dataset_import.add_argument('--dataset-only', action='store_true', default=False, help='skip manifest, import dataset only')
        parser_dataset_import.set_defaults(func=import_dataset)
        parser_dataset_export = subparsers_datasets.add_parser('export', help='Exports the dataset binary')
        parser_dataset_export.add_argument('uuid', help='uuid of the element to look at')
        parser_dataset_export.add_argument('--out', '-o', help='File to write to', type=argparse.FileType('w'))
        parser_dataset_export.set_defaults(func=dataset_get)




# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4

import ssl
import httplib
import json
import time
import copy
import urllib
from pprint import pprint
from fifo.helper import *

class Wiggle:
    def __init__(self):
        self._token = False
        self._apiEndpoint = '/api/2/'

    def init(self, host, user, pw, token, apiVersion, insecure):
        self.host = host
        self.insecure = insecure
        self.headers = {'content-type': 'application/json;charset=UTF-8',
                        'accept': 'application/json'}
        self._apiVersion = '2'
        if apiVersion:
            self._apiVersion = apiVersion
            self._apiEndpoint = '/api/' + apiVersion + '/'
        if token:
            if not self.set_token(token):
                self.connect(user, pw)
        else:
            self.connect(user, pw)

    def conn(self):
        if self.insecure:
            return httplib.HTTPConnection(self.host)
        else:
            return httplib.HTTPSConnection(self.host, context=ssl.create_default_context())

    def get_token(self):
        return self._token

    def set_token(self, token):
        self._token = token
        return self.get('sessions', token)

    def get(self, resource, entity, suffix='', rawResponse=False, headers=None):
        conn = self.conn()
        hdrs = headers or self.headers
        vprint('GET', self._apiEndpoint + resource + '/' + entity + suffix, '', hdrs)
        curlprint(self.host, 'GET', self._apiEndpoint + resource + '/' + entity + suffix, hdrs)
        conn.request('GET', self._apiEndpoint + resource + '/' + entity + suffix, '', hdrs)
        response = conn.getresponse()
        vprint('Status: ', response.status)
        if (response.status != 200):
            return False
        else:
            if rawResponse:
                return response
            else:
                return json.loads(response.read())

    def delete(self, resource, entity):
        conn = self.conn()
        vprint('DELETE', self._apiEndpoint + resource + '/' + entity, '', self.headers)
        curlprint(self.host, 'DELETE', self._apiEndpoint + resource + '/' + entity, self.headers)
        conn.request('DELETE', self._apiEndpoint + resource + '/' + entity, '', self.headers)
        response = conn.getresponse()
        vprint('Status: ', response.status)
        if (response.status >= 200 and response.status < 300):
            return True
        else:
            return False

    def delete_body(self, resource, entity, body):
        conn = self.conn()
        jbody = json.dumps(body)
        vprint('DELETE', self._apiEndpoint + resource + '/' + entity, jbody, self.headers)
        curlprint(self.host, 'DELETE', self._apiEndpoint + resource + '/' + entity, headers=self.headers, data=jbody)
        conn.request('DELETE', self._apiEndpoint + resource + '/' + entity, jbody, self.headers)
        response = conn.getresponse()
        vprint('Status: ', response.status)
        if (response.status >= 200 and response.status < 300):
            return True
        else:
            return False

    def delete_attr(self, resource, entity, attr):
        conn = self.conn()
        if isinstance(attr, str):
            attr = [attr]
        url = self._apiEndpoint + resource + '/' + entity + '/' + '/'.join(attr)
        vprint('DELETE', url, '', self.headers)
        curlprint(self.host, 'DELETE', url, headers=self.headers)
        conn.request('DELETE', url, '', self.headers)
        response = conn.getresponse()
        vprint('Status: ', response.status)
        if (response.status < 300 and response.status >= 300):
            return False
        else:
            return True

    def delete_attr_body(self, resource, entity, attr, body):
        conn = self.conn()
        if isinstance(attr, str):
            attr = [attr]
        url = self._apiEndpoint + resource + '/' + entity + '/' + '/'.join(attr)
        jbody = json.dumps(body)
        vprint('DELETE', url, jbody, self.headers)
        curlprint(self.host, 'DELETE', url, headers=self.headers, data=jbody)
        conn.request('DELETE', url, jbody, self.headers)
        response = conn.getresponse()
        vprint('Status: ', response.status)
        if (response.status < 300 and response.status >= 300):
            return False
        else:
            return True

    def put_attr(self, resource, entity, attr, body):
        conn = self.conn()
        if isinstance(attr, str):
            attr = [attr]
        jbody = json.dumps(body)
        url = self._apiEndpoint + resource + '/' + entity + '/' + '/'.join(attr)
        vprint('PUT', url, jbody, self.headers)
        curlprint(self.host, 'PUT', url, headers=self.headers, data=jbody)
        conn.request('PUT', url, jbody, self.headers)
        response = conn.getresponse()
        vprint('Status: ', response.status)
        if (response.status > 200 and response.status <= 400):
            return True
        else:
            return False

    def post_attr(self, resource, entity, attr, body):
        conn = self.conn()
        if isinstance(attr, str):
            attr = [attr]
        url = self._apiEndpoint + resource + '/' + entity + '/' + '/'.join(attr)
        jbody = json.dumps(body)
        vprint('POST', url, jbody, self.headers)
        curlprint(self.host, 'POST', url, headers=self.headers, data=jbody)
        conn.request('POST', url, jbody, self.headers)
        response = conn.getresponse()
        vprint('Status: ', response.status)
        if (response.status == 303):
            newurl = response.getheader('Location')
            conn = self.conn()
            conn.request('GET', newurl, '', self.headers)
            response = conn.getresponse()
            vprint('Status (after redirect): ', response.status)
            if (response.status == 404):
                time.sleep(5)
                conn = self.conn()
                conn.request('GET', newurl, '', self.headers)
                response = conn.getresponse()
                vprint('Status (after reload): ', response.status)
                if (response.status != 200):
                    return False
                else:
                    return json.loads(response.read())
            elif (response.status != 200):
                return False
            else:
                return json.loads(response.read())
        elif (response.status == 201):
            return 201
        elif (response.status == 200):
            return json.loads(response.read())
        else:
            return False

    def get_attr(self, resource, entity, attr):
        conn = self.conn()
        if isinstance(attr, str):
            attr = [attr]
        url = self._apiEndpoint + resource + '/' + entity + '/' + '/'.join(attr)
        vprint('GET', url, '', self.headers)
        curlprint(self.host, 'GET', url, headers=self.headers)
        conn.request('GET', url, '', self.headers)
        response = conn.getresponse()
        vprint('Status: ', response.status)
        if (response.status != 200):
            return False
        else:
            return json.loads(response.read())

    def put(self, resource, entity, body):
        conn = self.conn()
        if isinstance(entity, str):
            entity = [entity]
        url = self._apiEndpoint + resource + '/' + '/'.join(entity)
        jbody = json.dumps(body)
        vprint('PUT', url, jbody, self.headers)
        curlprint(self.host, 'PUT', url, headers=self.headers, data=jbody)
        conn.request('PUT', url, jbody, self.headers)
        response = conn.getresponse()
        vprint('Status: ', response.status)
        if (response.status == 201): # This was added for dryrun. Add body?
            return True
        if (response.status != 200):
            return False
        else:
            return json.loads(response.read())

    def put_bindata(self, resource, entity, file, suffix='', headers=None):
        conn = self.conn()
        if isinstance(entity, str):
            entity = [entity]
        url = self._apiEndpoint + resource + '/' + '/'.join(entity) + suffix
        hdrs = headers or self.headers
        vprint('PUT', url, '@' + file.name, hdrs)
        curlprint(self.host, 'PUT', url, headers=hdrs, upload=file.name)
        conn.request('PUT', url, file, hdrs)
        response = conn.getresponse()
        vprint('Status: ', response.status)
        if (response.status > 200 and response.status > 200):
            return response.status
        if (response.status != 200):
            return False
        else:
            return json.loads(response.read())

    def post(self, resource, body):
        conn = self.conn()
        if isinstance(resource, str):
            resource = [resource]
        url = self._apiEndpoint + '/'.join(resource)
        jbody = json.dumps(body)
        vprint('POST', url, jbody, self.headers)
        curlprint(self.host, 'POST', url, headers=self.headers, data=jbody)
        conn.request('POST', url, jbody, self.headers)
        response = conn.getresponse()
        vprint('Status: ', response.status)
        if (response.status == 303):
            newurl = response.getheader('Location')
            conn = self.conn()
            conn.request('GET', newurl, '', self.headers)
            response = conn.getresponse()
            if (response.status != 200):
                print response.status
                return False
            else:
                return json.loads(response.read())
        elif (response.status == 200):
            return json.loads(response.read())
        else:
            print response.status
            return False

    def post_bindata(self, resource, entity, file, headers=None):
        conn = self.conn()
        if isinstance(resource, str):
            resource = [resource]
        url = self._apiEndpoint + '/'.join(resource) + '/' + entity
        hdrs = headers or self.headers
        vprint('POST', url, '@' + file.name, hdrs)
        curlprint(self.host, 'POST', url, headers=hdrs, file=file.name, fileMode='binary')
        conn.request('POST', url, file, hdrs)
        response = conn.getresponse()
        vprint('Status: ', response.status)
        if (response.status == 303):
            newurl = response.getheader('Location')
            conn = self.conn()
            conn.request('GET', newurl, '', self.headers)
            response = conn.getresponse()
            if (response.status != 200):
                print response.status
                return False
            else:
                return json.loads(response.read())
        elif (response.status == 200):
            return json.loads(response.read())
        else:
            print response.status
            return False

    def list(self, resource):
        conn = self.conn()
        vprint('GET', self._apiEndpoint + resource, '', self.headers)
        curlprint(self.host, 'GET', self._apiEndpoint + resource, headers=self.headers)
        conn.request('GET', self._apiEndpoint + resource, '', self.headers)
        response = conn.getresponse()
        vprint('Status: ', response.status)
        if (response.status != 200):
            return False
        else:
            return json.loads(response.read())

    def full_list(self, resource, fields):
        conn = self.conn()
        hdrs = self.headers
        params = '?full-list=true'
        if fields != []:
            params = params + '&full-list-fields=' + ','.join(fields)
        vprint('GET', self._apiEndpoint + resource, '', hdrs)
        url = self._apiEndpoint + resource + params
        curlprint(self.host, 'GET', url, headers=hdrs)
        conn.request('GET', url, '', hdrs)
        response = conn.getresponse()
        vprint('Status: ', response.status)
        if (response.status != 200):
            return False
        else:
            return json.loads(response.read())

    def connect(self, user, pw):
        conn = self.conn()
        if self._apiVersion == '0.1.0':
            print "the 0.1.0 API is no longer supported!"
            exit(1)
        else:
            form = urllib.urlencode({'username' :user, 'password': pw, 'grant_type': 'password'})
            url = self._apiEndpoint + 'oauth/token'
            headers = copy.copy(self.headers)
            headers['content-type'] = 'application/x-www-form-urlencoded'
            vprint('POST', url, form, self.headers)
            curlprint(self.host, 'POST', url, headers=self.headers, data=form)
            conn.request('POST', url, form, self.headers)
            response = conn.getresponse()
            vprint('Status:', response.status)
            if (response.status == 200):
                resp = json.loads(response.read())
                self._token = resp['access_token']
                self.headers['authorization'] = 'Bearer ' + self._token
                return self._token
            else:
                return False


class Cloud:
    def __init__(self, wiggle):
        self._wiggle = wiggle
    def status(self):
        return self._wiggle.list('cloud')
    def connection(self):
        return self._wiggle.get('cloud', 'connection')

class Entity:
    def __init__(self, wiggle):
        self._resource = 'none'
        self._wiggle = wiggle
        self._fields = []

    def _put(self, uuid, body):
        uuid = self.uuid_by_name(uuid)
        return self._wiggle.put(self._resource, uuid, body)

    def _post(self, body):
        return self._wiggle.post(self._resource, body=body)

    def _put_file(self, entity, target, file):
        hdrs = dict(self._wiggle.headers)
        hdrs['Content-Type'] = 'application/x-gzip'
        return self._wiggle.put_bindata(self._resource, entity, headers=hdrs, suffix=target, file=file)

    def _post_file(self, entity, file):
        return self._wiggle.post_bindata(self._resource, entity, file=file)

    def _post_attr(self, uuid, entity, body):
        uuid = self.uuid_by_name(uuid)
        return self._wiggle.post_attr(self._resource, uuid, entity, body)

    def _put_attr(self, uuid, entity, body):
        uuid = self.uuid_by_name(uuid)
        return self._wiggle.put_attr(self._resource, uuid, entity, body)

    def _delete_attr(self, uuid, entity):
        uuid = self.uuid_by_name(uuid)
        return self._wiggle.delete_attr(self._resource, uuid, entity)

    def _delete_attr_body(self, uuid, entity, body):
        uuid = self.uuid_by_name(uuid)
        return self._wiggle.delete_attr_body(self._resource, uuid, entity, body)

    def _get_attr(self, uuid, attr):
        uuid = self.uuid_by_name(uuid)
        return self._wiggle.get_attr(self._resource, uuid, attr)

    def name_of(self, obj):
        return obj['name']

    def uuid_by_name(self, name):
        if is_uuid(name):
            return name
        else:
            for uuid in self.list():
                obj = self.get(uuid)
                if self.name_of(obj) == name:
                    return uuid
            return False

    def full_list(self, fields):
        return self._wiggle.full_list(self._resource, fields)

    def list(self):
        if not hasattr(self, '_fields'):
            self._fields = []
        return self._wiggle.full_list(self._resource, self._fields)

    def get(self, uuid):
        uuid = self.uuid_by_name(uuid)
        return self._wiggle.get(self._resource, uuid)

    def delete(self, uuid):
        uuid = self.uuid_by_name(uuid)
        return self._wiggle.delete(self._resource, uuid)

    def delete_body(self, uuid, body):
        uuid = self.uuid_by_name(uuid)
        return self._wiggle.delete_body(self._resource, uuid, body)

    def get_bindata(self, uuid, target, accept=''):
        uuid = self.uuid_by_name(uuid)
        binHeaders = dict(self._wiggle.headers)
        if accept:
            binHeaders['accept'] = accept
        else:
            del binHeaders['accept']
        return self._wiggle.get(self._resource, uuid, suffix=target, rawResponse=True, headers=binHeaders)

    def get_metadata(self, uuid):
        uuid = self.uuid_by_name(uuid)
        return self._wiggle.get_attr(self._resource, uuid, 'metadata')

    def set_metadata(self, uuid, path, k, v):
        uuid = self.uuid_by_name(uuid)
        path.insert(0, 'metadata')
        return self._wiggle.put_attr(self._resource, uuid, path, {k: v})

    def delete_metadata(self, uuid, path):
        uuid = self.uuid_by_name(uuid)
        path.insert(0, 'metadata')
        return self._wiggle.delete_attr(self._resource, uuid, path)

    def add_metadata_parser(self, subparsers):
        parser_metadata = subparsers.add_parser('metadata', help='Metadata commands')
        parser_metadata.add_argument('uuid', help='uuid of the element to look at')
        subparsers_metadata = parser_metadata.add_subparsers(help='Metadata commands')

        parser_mdata_get = subparsers_metadata.add_parser('get', help='get metadata')
        parser_mdata_get.set_defaults(func=mdata_get)

        parser_mdata_set = subparsers_metadata.add_parser('set', help='set metadata')
        parser_mdata_set.set_defaults(func=mdata_set)
        parser_mdata_set.add_argument('key', help='key of the metadata')
        group = parser_mdata_set.add_mutually_exclusive_group()
        group.add_argument('--integer', '-i', help='value is integer ', action='store_true')
        group.add_argument('--float', '-f', help='value is float', action='store_true')
        group.add_argument('--string', '-s', help='value is string', action='store_true')
        group.add_argument('--json', '-j', help='value is json', action='store_true')
        parser_mdata_set.add_argument('value', help='value to be set')
        parser_mdata_del = subparsers_metadata.add_parser('delete', help='deletes metadata')
        parser_mdata_del.set_defaults(func=mdata_delete)
        parser_mdata_del.add_argument('key', help='key of the metadata')


def mdata_get(args):
    e = args.endpoint.get(args.uuid)
    if not e or not e.has_key('metadata'):
        exit(1)
    services = e['metadata']
    print(json.dumps(services, sort_keys=True, indent=2, separators=(',', ': ')))

def mdata_set(args):
    keys = args.key.split('.')
    k = keys.pop()
    value = args.value
    if args.integer:
        value = int(args.value)
    elif args.float:
        value = float(args.value)
    elif args.json:
        value = json.loads(args.value)
    if args.endpoint.set_metadata(args.uuid, keys, k, value):
        print 'Metadata successfully updated!'
    else:
        print 'Failed to update metadata!'
        exit(1)

def mdata_delete(args):
    keys = args.key.split('.')
    if args.endpoint.delete_metadata(args.uuid, keys):
        print 'Metadata successfully updated!'
    else:
        print 'Failed to update metadata!'
        exit(1)



# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4

from .wiggle import Entity
from fifo.helper import *
import re

pkg_fmt = {
    'uuid':
    {'title': 'UUID', 'len': 36, 'fmt': '%36s', 'get': lambda e: d(e, ['uuid'])},
    'name':
    {'title': 'Name', 'len': 10, 'fmt': '%-10s', 'get': lambda e: d(e, ['name'])},
    'cpu_cap':
    {'title': 'CPU cap', 'len': 10, 'fmt': '%-10s', 'get': lambda e: str(d(e, ['cpu_cap'])) + '%'},
    'quota':
    {'title': 'Quota', 'len': 10, 'fmt': '%-10s', 'get': lambda e: str(d(e, ['quota'])) + ' GB'},
    'ram':
    {'title': 'RAM', 'len': 10, 'fmt': '%-10s', 'get': lambda e: str(d(e, ['ram'])) + ' MB'},
}


def create(args):
    mb = 1024*1024
    gb = mb * 1024
    reqs = []
    if args.requirement:
        reqs = map(package_rule, args.requirement)
    if args.random:
        reqs += map(package_random, args.random)
    if args.scale:
        reqs += map(package_scale, args.scale)
    if args.memory < mb:
        msg = 'The minimum amount of memory for a package is 1MB but %r byte were reqested' % args.memory
        raise argparse.ArgumentTypeError(msg)
    if args.quota < gb:
        msg = 'The minimum amount of quote for a package is 1GB but %r byte were reqested' % args.quota
        raise argparse.ArgumentTypeError(msg)
    res = args.endpoint.create(args.name, int(args.memory/mb), int(args.quota/gb), args.cpu_cap, reqs)
    if args.p:
        if res:
            print res['uuid']
        else:
            exit(1)
    else:
        if res:
            print 'Pacakge successfully created: %s' % res['uuid']
        else:
            print 'Package creation failed: %r' % res
            exit(1)


def byte_size(string):
    scale_map = {
        'B': 1,
        'KB': 1024,
        'MB': 1024*1024,
        'GB': 1024*1024*1024,
        'TB': 1024*1024*1024*1024,
        'EB': 1024*1024*1024*1024*1024,
        'PB': 1024*1024*1024*1024*1024*1024,
    }
    p = re.compile('(\d+)(B|KB|MB|GB|TB|EB|PB)?')
    m = p.match(string)
    if not m:
        msg = '%r is not byte size. Use <number>[B|KB|MB|GB|TB|PB|EB]' % string
        raise argparse.ArgumentTypeError(msg)
    value = int(m.group(1))
    scale = m.group(2) or 'B'
    return value*scale_map[scale]

def package_rule(rule):
    weight = mk_weight(rule[0])
    attr = rule[1]
    cond = mk_cond(rule[2])
    val = mk_value(rule[3])
    return {
        'weight': weight,
        'condition': cond,
        'attribute': attr,
        'value': val
    }


def package_scale(scale):
    return {
        'weight': 'scale',
        'attribute': scale[0],
        'low': int(scale[1]),
        'high': int(scale[2])
    }

def package_random(rand):
    return {
        'weight': 'random',
        'low': int(scale[0]),
        'high': int(scale[1])
    }


def mk_weight(weight):
    if weight == 'must':
        return weight
    elif weight == 'cant':
        return weight
    else:
        return int(weight)

def mk_cond(cond):
    valid = ['>=', '|>', '=<', '<', '=:=', '=/=', 'subset', 'superset', 'disjoint', 'element']
    if not cond in valid:
        msg = '%r is not a valid condition, must be one of %r' % (cond, valid)
        raise argparse.ArgumentTypeError(msg)
    return cond;

def mk_value(val):
    num = re.compile('\d+')
    if num.match(val):
        return int(val)
    return val

class Package(Entity):
    def __init__(self, wiggle):
        self._wiggle = wiggle
        self._resource = 'packages'

    def create(self, name, ram, quota, cpu_cap, reqs):
        specs = {
            'name': name,
            'ram': ram,
            'quota': quota,
            'cpu_cap': cpu_cap
        }
        if reqs != []:
            specs['requirements'] = reqs
        return self._post(specs)

    def make_parser(self, subparsers):
        parser_pkgs = subparsers.add_parser('packages', help='package related commands')
        parser_pkgs.set_defaults(endpoint=self)
        subparsers_pkgs = parser_pkgs.add_subparsers(help='package commands')
        self.add_metadata_parser(subparsers_pkgs)
        parser_pkgs_list = subparsers_pkgs.add_parser('list', help='lists packages')
        parser_pkgs_list.add_argument('--fmt', action=ListAction, default=['uuid', 'name', 'ram', 'cpu_cap', 'quota'])
        parser_pkgs_list.add_argument('-H', action='store_false')
        parser_pkgs_list.add_argument('-p', action='store_true')
        parser_pkgs_list.set_defaults(func=show_list,
                                      fmt_def=pkg_fmt)
        parser_pkgs_get = subparsers_pkgs.add_parser('get', help='gets a package')
        parser_pkgs_get.add_argument('uuid')
        parser_pkgs_get.set_defaults(func=show_get)
        parser_pkgs_delete = subparsers_pkgs.add_parser('delete', help='deletes a package')
        parser_pkgs_delete.add_argument('uuid')
        parser_pkgs_delete.set_defaults(func=show_delete)
        parser_pkgs_create = subparsers_pkgs.add_parser('create', help='creates a new package')
        parser_pkgs_create.add_argument('-p', action='store_true',
                                        help='Output only shows the UUID no additional comments.')
        parser_pkgs_create.add_argument('--memory', '-m', required=True, type=byte_size,
                                        help='Amount of memory for the package.')
        parser_pkgs_create.add_argument('--quota', '-q', required=True, type=byte_size,
                                        help='Amount of quota for the package.')
        parser_pkgs_create.add_argument('--cpu_cap', '-c', required=True, type=int,
                                        help='Percentage of cores for the pacakge 100 = 1core.')
        parser_pkgs_create.add_argument('--requirement', '-r', action='append', nargs=4,
                                        help='Requirement rule <weight> <attr> <cond> <val>')
        parser_pkgs_create.add_argument('--scale', '-s', action='append', nargs=3,
                                        help='Scale rule <attr> <low> <hight>')
        parser_pkgs_create.add_argument('--random', '-R', action='append', nargs=2,
                                        help='Ramdom rule <low> <hight>')
        parser_pkgs_create.add_argument('name')
        parser_pkgs_create.set_defaults(func=create)



# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4

from .wiggle import Entity
from fifo.helper import *


grouping_fmt = {
    'uuid':
    {'title': 'UUID', 'len': 36, 'fmt': '%36s', 'get': lambda e: d(e, ['uuid'])},
    'name':
    {'title': 'Name', 'len': 10, 'fmt': '%-10s', 'get': lambda e: d(e, ['name'])},
    'type':
    {'title': 'Name', 'len': 10, 'fmt': '%-10s', 'get': lambda e: d(e, ['type'])},
    'elements':
    {'title': 'elements', 'len': 36, 'fmt': '%36s', 'get': lambda e: ','.join(d(e, ['elements']))},
    'groupings':
    {'title': 'groupings', 'len': 36, 'fmt': '%36s', 'get': lambda e: ','.join(d(e, ['groupings']))},
 }


def create_grouping(args):
    res = args.endpoint.create(args.name, args.type)
    if res:
        print 'New ' + args.type + ' created: ' + res['uuid']
    else:
        print 'Failed to create ' + args.type
        exit(1)

def grouping_add(args):
    main = args.endpoint.get(args.uuid)
    if main['type'] != 'stack':
        print 'Only clusters can be added to stacks.'
        exit(1)
    res = args.endpoint.get(args.target)
    if not res:
        print 'Could not find cluster ' + args.target
        exit(1)
    if not res['type'] == 'cluster':
        print args.target + ' is not a cluster.'
        exit(1)
    res = args.endpoint.add_element(args.uuid, args.target)
    if res:
        print 'Successfully added cluster ' + args.target + '.'
    else:
        print 'Failed to add added cluster ' + args.target + '.'
        exit(1)

def grouping_remove(args):
    main = args.endpoint.get(args.uuid)
    if main['type'] != 'stack':
        print 'Only clusters can be added to stacks.'
        exit(1)
    if main['type'] == 'cluster' and args.type == 'cluster':
        print 'Only VMs can be added to clusters.'
        exit(1)
    res = args.endpoint.delete_element(args.uuid, args.target)
    if res:
        print 'Successfully removed ' + args.type + ' ' + args.target + ' from ' + args.uuid
    else:
        print 'Failed to removed ' + args.type + ' from ' + args.uuid
        exit(1)


class Grouping(Entity):
    def __init__(self, wiggle):
        self._wiggle = wiggle
        self._resource = 'groupings'

    def create(self, name, type):
        return self._post({'name': name, 'type': type})

    def add_element(self, uuid, element):
        return self._put_attr(uuid, ['elements', element], {})

    def delete_element(self, uuid, element):
        return self._delete_attr(uuid, ['elements', element])

    def make_parser(self, subparsers):
        parser_groupings = subparsers.add_parser('groupings', help='grouping related commands')
        parser_groupings.set_defaults(endpoint=self)
        subparsers_groupings = parser_groupings.add_subparsers(help='grouping commands')
        self.add_metadata_parser(subparsers_groupings)
        parser_groupings_list = subparsers_groupings.add_parser('list', help='lists groupings')
        parser_groupings_list.add_argument('--fmt', action=ListAction,
                                          default=['uuid', 'name', 'type'])
        parser_groupings_list.add_argument('-H', action='store_false')
        parser_groupings_list.add_argument('-p', action='store_true')
        parser_groupings_list.set_defaults(func=show_list,
                                          fmt_def=grouping_fmt)
        parser_groupings_get = subparsers_groupings.add_parser('get', help='gets a grouping')
        parser_groupings_get.add_argument('uuid')
        parser_groupings_get.set_defaults(func=show_get)
        parser_groupings_delete = subparsers_groupings.add_parser('delete', help='deletes a grouping')
        parser_groupings_delete.add_argument('uuid')
        parser_groupings_delete.set_defaults(func=show_delete)

        parser_groupings_create = subparsers_groupings.add_parser('create', help='creates a new grouping')
        parser_groupings_create.add_argument('--type', '-t', required=True, type=str, choices=['stack', 'cluster'])
        parser_groupings_create.add_argument('name')
        parser_groupings_create.set_defaults(func=create_grouping)

        parser_groupings_add = subparsers_groupings.add_parser('add', help='Adds elements to a crouping')
        parser_groupings_add.add_argument('uuid')
        parser_groupings_add.add_argument('type', type=str, choices=['cluster'])
        parser_groupings_add.add_argument('target')
        parser_groupings_add.set_defaults(func=grouping_add)

        parser_groupings_remove = subparsers_groupings.add_parser('remove', help='Removes elements to a crouping')
        parser_groupings_remove.add_argument('uuid')
        parser_groupings_remove.add_argument('type', type=str, choices=['vm', 'cluster'])
        parser_groupings_remove.add_argument('target')
        parser_groupings_remove.set_defaults(func=grouping_remove)
